STREAMLANA_ANONYMOUS_PG_NAMES = "STREAMLANA_ANONYMOUS_PG_NAMES"
STREAMLANA_URL_PG_NAME_PREFIX = "STREAMLANA_URL_PG_NAME_PREFIX"
# used for debugging
STREAMLANA_DEBUG_STATE = "STREAMLANA_DEBUG_STATE"
STREAMLANA_DEBUG_DF = "STREAMLANA_DEBUG_DF"
STREAMLANA_DEBUG_BORDER_ALL_WIDGETS = "STREAMLANA_BORDER_ALL_WIDGETS"

# used for tests
STREAMLANA_TEST_USE_APP_INMEM_STATE = "STREAMLANA_USE_APP_INMEM_STATE"
